@@ -173,7 +173,7 @@ sub run {
                 m/^changelog\.debian(?:\.gz)$/i;
             }
             @doc_files
-          ) {
+        ) {
             tag 'wrong-name-for-changelog-of-native-package',
               "usr/share/doc/$pkg/$chg";
         } else {
@@ -189,19 +189,13 @@ sub run {
             # everything is fine
         } else {
             # search for changelogs with wrong file name
-            my $found = 0;
             for (@doc_files) {
                 if (m/^change/i and not m/debian/i) {
                     tag 'wrong-name-for-upstream-changelog',
                       "usr/share/doc/$pkg/$_";
-                    $found = 1;
                     last;
                 }
             }
-            if (not $found) {
-                tag 'no-upstream-changelog'
-                  unless $info->is_pkg_class('transitional');
-            }
         }
 
         # 2. check for Debian changelog
@@ -212,7 +206,7 @@ sub run {
                 m/^changelog\.debian(?:\.gz)?$/i;
             }
             @doc_files
-          ) {
+        ) {
             tag 'wrong-name-for-debian-changelog-file',
               "usr/share/doc/$pkg/$chg";
         } else {
@@ -320,6 +314,7 @@ sub run {
                 tag 'latest-debian-changelog-entry-without-new-version'
                   unless versions_gt($first_version, $second_version)
                   or $entries[0]->Changes =~ /backport/i
+                  or $entries[0]->Distribution =~ /-security$/i
                   or $entries[0]->Source ne $entries[1]->Source;
                 tag 'latest-debian-changelog-entry-changed-to-native'
                   if $native_pkg and $second_version =~ m/-/;
@@ -346,6 +341,13 @@ sub run {
             my $second_debian =substr $second_version,length($second_upstream);
             $second_debian =~ s/-([^-]+)$/$1/ if length($second_debian) > 0;
 
+            if ($first_version =~ /((?:[~+]deb\d+u1|\+nmu1)$)/) {
+                my $expected = substr($first_version, 0, -length($1));
+                tag 'changelog-file-missing-explicit-entry',
+                  "$second_version -> $expected (missing) -> $first_version"
+                  unless $second_version eq $expected;
+            }
+
             if (    $first_upstream eq $second_upstream
                 and $entries[0]->Source eq $entries[1]->Source) {
                 if ($entries[0]->Changes
@@ -369,6 +371,14 @@ sub run {
                 }
             }
 
+            my $changes = $group->get_changes_processable;
+            if ($changes) {
+                my $chgs_dist = lc($changes->info->field('distribution', ''));
+                tag 'changelog-distribution-does-not-match-changes-file',
+                  "($first_dist != $chgs_dist)"
+                  if $first_dist ne $chgs_dist;
+            }
+
             my ($first_epoch) = ($first_version =~ /^([^:]+):/, '(none)');
             my ($second_epoch) = ($second_version =~ /^([^:]+):/, '(none)');
             if ($first_epoch and $second_epoch ne $first_epoch) {
@@ -411,7 +421,7 @@ sub run {
         if (
             $changes =~ /uploads? \s+ to \s+
                             (?'intended'testing|unstable|experimental|sid)/xi
-          ){
+        ){
             my $intended = lc($+{intended});
             if($intended eq 'sid') {
                 $intended = 'unstable';
@@ -432,7 +442,8 @@ sub run {
         my $changesempty = $changes;
         $changesempty =~ s,\W,,gms;
         if (length($changesempty)==0) {
-            tag 'changelog-empty-entry';
+            tag 'changelog-empty-entry'
+              unless $entry->Distribution eq 'UNRELEASED';
         }
 
         my $closes = $entry->Closes;
@@ -548,7 +559,7 @@ sub check_dch {
             || m/closes:\s*(?:bug)?\#?\s?\d+
               (?:,\s*(?:bug)?\#?\s?\d+)*
               (?:,\s*(((?:bug)?\#?\s?\d*)[[:alpha:]]\w*))/iox
-          ) {
+        ) {
             tag 'wrong-bug-number-in-closes', "l$.:$1" if $2;
         }
 
