@@ -28,7 +28,7 @@ use List::MoreUtils qw(none);
 use Lintian::Data;
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(drain_pipe internal_error is_string_utf8_encoded open_gz
-  signal_number2name strip normalize_pkg_path slurp_entire_file);
+  signal_number2name strip normalize_pkg_path);
 use Lintian::SlidingWindow;
 
 use constant BLOCKSIZE => 16_384;
@@ -304,6 +304,7 @@ sub run {
     my %linked_against_libvga;
     my @devhelp;
     my @devhelp_links;
+    my @bin_binaries;
 
     # X11 bitmapped font directories under /usr/share/fonts/X11 in which we've
     # seen files.
@@ -423,6 +424,11 @@ sub run {
         my $link = $file->link;
         my $finfo = $file->file_info;
 
+        # If we have a /usr/sbin/foo, check for references to
+        # /usr/bin/foo
+        push(@bin_binaries, "/$1bin/$2")
+          if $file->is_file and $fname =~ m,^(usr/)?sbin/(.+),;
+
         $arch_dep_files = 1
           if not $file->is_dir
           and $fname !~ m,^usr/share/,o
@@ -451,9 +457,10 @@ sub run {
             tag $tag, $file if $fname =~ m/$regex/;
         }
 
-        tag 'package-contains-real-file-outside-usr', $file
+        tag 'package-contains-usr-unmerged-pathnames', $file
           if $fname =~ m,^(?:bin|sbin|lib.*)/.+$,
           and not $file->is_symlink
+          and not $file->is_dir
           and ($link // '') !~ m,^usr/$fname$,;
 
         if ($file->is_hardlink) {
@@ -1345,7 +1352,7 @@ sub run {
             and not $fname =~ m{ \. (?:
                   # Common "non-license" file extensions...
                    el|[ch]|cc|p[ylmc]|[hu]i|p_hi|html|php|rb|xpm
-                     |png|jpe?g|gif|svg|dtd|mk|lisp|yml|rs|ogg
+                     |png|jpe?g|gif|svg|dtd|mk|lisp|yml|rs|ogg|xbm
                ) \Z}xsm
             and not $fname=~ m,^usr/share/zope/Products/.*\.(?:dtml|pt|cpt)$,
             and not $fname =~ m,/under\S+License\.docbook$,
@@ -1519,10 +1526,14 @@ sub run {
 
             # ---------------- contents checks
             my %checks
-              = get_checks_for_file($info, $file, $source_pkg, $build_path);
+              = get_checks_for_file($info, $file, $source_pkg,
+                $build_path, @bin_binaries);
 
             if (%checks) {
-                my $strings = slurp_entire_file($info->strings($file));
+                my $stringsfd = $info->strings($file);
+                my $strings = do { local $/; <$stringsfd> };
+                close($stringsfd)
+                  or warn "Error closing strings fd: $!";
 
                 foreach my $tag (sort keys %checks) {
                     my $regex = $checks{$tag};
@@ -1549,6 +1560,10 @@ sub run {
                 foreach my $taboo ($DOCUMENTATION_FILE_REGEX->all) {
                     my $regex = $DOCUMENTATION_FILE_REGEX->value($taboo);
                     if($file->basename =~ m{$regex}xi) {
+                        # No need for dh-r packages to automatically
+                        # create overrides if we just allow them all to
+                        # begin with.
+                        next if $file->dirname =~ m{^usr/lib/R/site-library/};
                         # see #904852
                         next if $file->dirname =~ m{templates?(?:\.d)?/};
                         next
@@ -1641,10 +1656,12 @@ sub run {
                     tag 'duplicate-font-file', "$fname also in", $font_owner
                       if ($pkg ne $font_owner and $type ne 'udeb');
                 } elsif ($pkg !~ m/^(?:[ot]tf|t1|x?fonts)-/) {
-                    tag 'font-in-non-font-package', $file;
+                    tag 'font-in-non-font-package', $file
+                      unless $fname =~ m,^usr/lib/R/site-library/,;
                 }
                 tag 'font-outside-font-dir', $file
-                  unless $fname =~ m,^usr/share/fonts/,;
+                  unless $fname =~ m,^usr/share/fonts/,
+                  or $fname =~ m,^usr/lib/R/site-library/,;
 
                 my $finfo = $file->file_info;
                 if ($finfo =~ m/PostScript Type 1 font program data/) {
@@ -2049,7 +2066,8 @@ sub run {
         next if not $file or not $file->is_open_ok;
 
         my %checks
-          = get_checks_for_file($info, $file, $pkg_section, $build_path);
+          = get_checks_for_file($info, $file, $source_pkg, $build_path,
+            @bin_binaries);
         my $fd2 = $file->open;
         while (<$fd2>) {
             foreach my $tag (sort keys %checks) {
@@ -2360,7 +2378,7 @@ sub detect_privacy_breach {
 }
 
 sub get_checks_for_file {
-    my ($info, $file, $source_pkg, $build_path) = @_;
+    my ($info, $file, $source_pkg, $build_path, @bin_binaries) = @_;
     my %checks;
 
     return %checks if $source_pkg eq 'lintian';
@@ -2378,7 +2396,12 @@ sub get_checks_for_file {
       and $info->field('section', '') ne 'debian-installer'
       and none { $_ eq $source_pkg } qw(base-files dpkg lintian);
 
-    $checks{'file-references-package-build-path'} = $build_path if $build_path;
+    $checks{'file-references-package-build-path'} = $build_path
+      if defined $build_path && $build_path =~ m,^/.+,g;
+
+    # If we have a /usr/sbin/foo, check for references to /usr/bin/foo
+    $checks{'bin-sbin-mismatch'} = '(' . join('|', @bin_binaries) . ')'
+      if @bin_binaries;
 
     return %checks;
 }
